#!/usr/bin/env python
# -*- mode: python; coding: koi8-r -*-


import string, os

import gtk, gobject
from gtk import TRUE, FALSE

from pybookreader.bookmarks import *
from pybookreader.utils import *
from pybookreader.bookbuffer import *
#from pybookreader.filters import *
import pybookreader.filters

##----------------------------------------------------------------------
##----------------------------------------------------------------------


class BookmarksWindow(gtk.Window):

    width = 600
    height = 400


    COLUMN_FOLDER     = 0
    COLUMN_FILENAME   = 1
    COLUMN_AUTHOR     = 2
    COLUMN_NAME       = 3
    COLUMN_PERCENT    = 4
    COLUMN_DECRIPTION = 5
    COLUMN_PATH       = 6 # invisible: path to file
    COLUMN_MARK       = 7 # invisible: mark number


    def __init__(self, bookmarks):


        gtk.Window.__init__(self)

        self.categories_list = bookmarks.categories_list
        self.bookmarks = bookmarks

        self.set_title('%s: Bookmarks manager' % program_name)

        self.connect('destroy', self.destroy_cb)
        self.connect('size_allocate', self.size_allocate_cb)

        # -- top level vbox --

        top_vbox =  gtk.VBox()
        self.add(top_vbox)

        # -- bbox --

        bbox = gtk.HButtonBox()
        top_vbox.pack_start(bbox, expand=FALSE)


        bbox.set_border_width(0)
        bbox.set_layout(gtk.BUTTONBOX_START)
        bbox.set_spacing(0)
        #bbox.set_property('homogeneous', FALSE) # don't work, why?
        #bbox.set_homogeneous(TRUE)

        button = gtk.Button('_Add file(s)')
        button.set_property('relief', gtk.RELIEF_NONE)
        bbox.add(button)
        button.connect('clicked', self.add_file_cb)


        button = gtk.Button('_New category')
        button.set_property('relief', gtk.RELIEF_NONE)
        bbox.add(button)
        button.connect('clicked', self.add_category_cb)

        button = gtk.Button('_Delete')
        button.set_property('relief', gtk.RELIEF_NONE)
        bbox.add(button)
        button.connect('clicked', self.delete_clicked_cb)

        button = gtk.Button('A_pply')
        button.set_property('relief', gtk.RELIEF_NONE)
        bbox.add(button)
        button.connect('clicked', self.apply_cliced_cb)

        button = gtk.Button('_Load')
        button.set_property('relief', gtk.RELIEF_NONE)
        bbox.add(button)
        button.connect('clicked', lambda w: self.emit_load_file())

        button = gtk.Button('_Close')
        button.set_property('relief', gtk.RELIEF_NONE)
        bbox.add(button)
        button.connect('clicked', self.close_clicked_cb)


        self.paned = gtk.VPaned()
        top_vbox.add(self.paned)

        # -- list --

        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled_window.set_shadow_type(gtk.SHADOW_IN)

        self.paned.pack1(scrolled_window)


        # ...
        treeview = gtk.TreeView()
        #treeview.set_search_column(self.COLUMN_FILENAME) # ???
        treeview.set_rules_hint(TRUE) #   
        scrolled_window.add(treeview)


        # category
        column = gtk.TreeViewColumn('Category', gtk.CellRendererText(),
                                    text=self.COLUMN_FOLDER)
        column.set_resizable(TRUE)
        treeview.append_column(column)
        # filename
        column = gtk.TreeViewColumn('File', gtk.CellRendererText(),
                                    text=self.COLUMN_FILENAME)
        column.set_resizable(TRUE)
        treeview.append_column(column)
        # author
        column = gtk.TreeViewColumn('Author', gtk.CellRendererText(),
                                    text=self.COLUMN_AUTHOR)
        column.set_resizable(TRUE)
        treeview.append_column(column)
        # name
        column = gtk.TreeViewColumn('Name', gtk.CellRendererText(),
                                    text=self.COLUMN_NAME)
        column.set_resizable(TRUE)
        treeview.append_column(column)
        # percent
        column = gtk.TreeViewColumn('%', gtk.CellRendererText(),
                                    text=self.COLUMN_PERCENT)
        column.set_resizable(TRUE)
        treeview.append_column(column)
        # description
        column = gtk.TreeViewColumn('Description', gtk.CellRendererText(),
                                    text=self.COLUMN_DECRIPTION)
        column.set_resizable(TRUE)
        treeview.append_column(column)

        treeview.connect('row-activated', lambda w, row, col:
                         self.emit_load_file())

        # -- treview selection --
        selection = treeview.get_selection()
        selection.connect('changed', self.treeview_selected_cb)
        selection.set_mode(gtk.SELECTION_MULTIPLE)


        self.treeview = treeview

        # -- notebook --

        notebook = gtk.Notebook()
        self.paned.pack2(notebook)

        # -- vbox --

        vbox=gtk.VBox()

        label = gtk.Label('')
        label.set_text_with_mnemonic('De_scription')
        notebook.append_page(vbox, label)

        # -- table --

        table = gtk.Table(5, 2)
        vbox.pack_start(table, expand=FALSE, fill=TRUE, padding=0)

        label = gtk.Label('File: ')
        label.set_justify(gtk.JUSTIFY_LEFT)
        table.attach(label, 0, 1, 0, 1, xoptions=gtk.FILL,
                     xpadding=4, ypadding=4)
        self.file_label = gtk.Label('')
        table.attach(self.file_label, 1, 4, 0, 1, xpadding=4, ypadding=4)



##         label = gtk.Label('Filter: ')
##         self.filters_button = gtk.Button('Filters')
##         table.attach(self.filters_button, 2, 4, 0, 1, xpadding=4, ypadding=4)
##         self.filters_button.connect('clicked', self.filters_cliced_cb)






        label = gtk.Label('Ca_tegory: ')
        label.set_justify(gtk.JUSTIFY_LEFT)
        label.set_use_underline(TRUE)
        table.attach(label, 0, 1, 1, 2, xoptions=gtk.FILL,
                     xpadding=4, ypadding=4)
        self.category_menu = gtk.OptionMenu()
        table.attach(self.category_menu, 1, 2, 1, 2, xpadding=4, ypadding=4)
        label.set_mnemonic_widget(self.category_menu)

        label = gtk.Label('_Encoding: ')
        label.set_justify(gtk.JUSTIFY_LEFT)
        label.set_use_underline(TRUE)
        table.attach(label, 2, 3, 1, 2, xoptions=gtk.FILL,
                     xpadding=4, ypadding=4)
        self.encoding_menu = gtk.OptionMenu()
        table.attach(self.encoding_menu, 3, 4, 1, 2, xpadding=4, ypadding=4)
        label.set_mnemonic_widget(self.encoding_menu)

        label = gtk.Label('A_uthor: ')
        label.set_justify(gtk.JUSTIFY_LEFT)
        label.set_use_underline(TRUE)
        table.attach(label, 0, 1, 2, 3, xoptions=gtk.FILL,
                     xpadding=4, ypadding=4)
        self.author_entry = gtk.Entry()
        table.attach(self.author_entry, 1, 2, 2, 3, xpadding=4, ypadding=4)
        label.set_mnemonic_widget(self.author_entry)
        self.author_entry.connect('activate', lambda w: self.change_apply())

        label = gtk.Label('Na_me: ')
        label.set_justify(gtk.JUSTIFY_LEFT)
        label.set_use_underline(TRUE)
        table.attach(label, 2, 3, 2, 3, xoptions=gtk.FILL,
                     xpadding=4, ypadding=4)
        self.name_entry = gtk.Entry()
        table.attach(self.name_entry, 3, 4, 2, 3, xpadding=4, ypadding=4)
        label.set_mnemonic_widget(self.name_entry)
        self.name_entry.connect('activate', lambda w: self.change_apply())

        # -- description --

        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled_window.set_border_width(4)
        scrolled_window.set_shadow_type(gtk.SHADOW_IN)

        vbox.pack_end(scrolled_window)

        self.description_view = gtk.TextView()
        scrolled_window.add(self.description_view)
        self.description_buffer = gtk.TextBuffer()
        self.description_view.set_buffer(self.description_buffer)

        self.description_view.set_wrap_mode(gtk.WRAP_WORD)

        # -- preview --

        label = gtk.Label('')
        label.set_text_with_mnemonic('Pre_view') # FIXME: name

        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_border_width(4)
        scrolled_window.set_shadow_type(gtk.SHADOW_IN)
        notebook.append_page(scrolled_window, label)

        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        self.text_view = gtk.TextView()
        scrolled_window.add(self.text_view)

        self.text_buffer = BookBuffer()
        self.text_view.set_buffer(self.text_buffer)

        self.text_view.set_editable(FALSE)
        self.text_view.set_cursor_visible(FALSE)
        self.text_view.set_wrap_mode(gtk.WRAP_WORD)






        # -- filters --

        label = gtk.Label('')
        label.set_text_with_mnemonic('Filters')




        filters_table =  gtk.Table()
        notebook.append_page(filters_table, label)

        i=0
        self.filters_check_list = []
        for fltr in pybookreader.filters.filters:

            check = gtk.CheckButton(pybookreader.filters.filters[fltr])
            #check.set_active(parent.text_widget.scroll_type[i][0])
            check.connect('clicked', self.filters_toggled, fltr)
            filters_table.attach(check, 0, 4, i, i+1,
                                 xoptions=gtk.FILL, yoptions=gtk.FILL,
                                 xpadding=4, ypadding=4)
            self.filters_check_list.append((fltr, check))
            i+=1

        label = gtk.Label('External filter: ')
        filters_table.attach(label, 0, 1, i, i+1,
                             xoptions=gtk.FILL, yoptions=gtk.FILL,
                             xpadding=4, ypadding=4)

        self.ext_filter_label = gtk.Label('None')
        filters_table.attach(self.ext_filter_label, 1, 2, i, i+1,
                             xoptions=gtk.FILL, yoptions=gtk.FILL,
                             xpadding=4, ypadding=4)

        button = gtk.Button('Change')
        filters_table.attach(button, 2, 3, i, i+1,
                             xoptions=gtk.FILL, yoptions=gtk.FILL,
                             xpadding=4, ypadding=4)
        button.connect('clicked', self.ext_filter_clicked_cb)


        # -- charsets --
        self.encoding_submenu = gtk.Menu()
        for i in charsets_list:
            item = gtk.MenuItem(i)
            self.encoding_submenu.append(item)
            item.connect_object('activate', self.encodind_submenu_activate_cb, i)
            item.show()
        self.encoding_menu.set_menu(self.encoding_submenu)

        # -- categories --
        self.update_category_menu()

        self.update_treeview()

##----------------------------------------------------------------------

    def get_selected_list(self):

        sel = self.treeview.get_selection()

        selected_list = []
        sel.selected_foreach(lambda model, path, iter:
                             selected_list.append((model, path, iter)))

        return selected_list

##----------------------------------------------------------------------

    def encodind_submenu_activate_cb(self, enc):

        selected_list = self.get_selected_list()

        if len(selected_list) != 1:
            return

        model, path, iter = selected_list[0]
        filename = model.get_value(iter, self.COLUMN_PATH)
        b = self.bookmarks.get_book(filename)

        if b:

            b.encoding = enc

            # FIXME: change description?

            self.treeview_show_selected(selected_list[0])



##----------------------------------------------------------------------

    def category_submenu_activate_cb(self, new_cat):

        selected_list = self.get_selected_list()

        if len(selected_list) != 1: return

        model, path, iter = selected_list[0]
        filename = model.get_value(iter, self.COLUMN_PATH)
        if not filename: return

        self.bookmarks.move_book(filename, new_cat)

        self.update_treeview()

##----------------------------------------------------------------------

    def treeview_selected_cb(self, sel):

        selected_list = self.get_selected_list()

        if len(selected_list) == 1:
            self.treeview_show_selected(selected_list[0])
        else:
            self.clear_view()

##----------------------------------------------------------------------

    def treeview_show_selected(self, args):
        model, path, iter = args

        filename = model.get_value(iter, self.COLUMN_PATH)
        mark = model.get_value(iter, self.COLUMN_MARK)

        found = 0
        b = self.bookmarks.get_book(filename)
        if b:

            self.file_label.set_text(b.filename)

            if mark:
                m = self.bookmarks.get_mark(b, int(mark))
                if m:

                    if m.author:
                        self.author_entry.set_text(m.author)
                    else:
                        self.author_entry.set_text(b.author)
                    if m.name:
                        self.name_entry.set_text(m.name)
                    else:
                        self.name_entry.set_text(b.name)
                    self.description_buffer.set_text(m.description)

                    # update text preview
                    self.text_buffer.set_book(b)
                    self.text_buffer.preview(m.offset)

            else:

                self.author_entry.set_text(b.author)
                self.name_entry.set_text(b.name)
                self.description_buffer.set_text(b.description)

                # update text preview
                self.text_buffer.set_book(b)
                self.text_buffer.preview()

            # update encoding menu
            n = 0
            for enc in charsets_list:
                if b.encoding == enc:
                    break
                n = n+1
            # bug ?
            self.encoding_menu.remove_menu()
            self.encoding_submenu.set_active(n)
            self.encoding_menu.set_menu(self.encoding_submenu)

            # update category menu
            n = 0
            for cat in self.categories_list:
                if b in self.categories_list[cat]:
                    break
                n = n+1
            self.category_menu.remove_menu()
            self.category_submenu.set_active(n)
            self.category_menu.set_menu(self.category_submenu)

            # update filters
            for c in self.filters_check_list:
                c[1].set_active(FALSE)
                if b.internalfilters:
                    for f in b.internalfilters:
                        if f == c[0]:
                            c[1].set_active(TRUE)
                            break


            self.ext_filter_label.set_text(str(b.externalfilter))

            found = 1

        if not found: self.clear_view()

##----------------------------------------------------------------------

    def destroy_cb(self, w):
        pass

##----------------------------------------------------------------------

    def size_allocate_cb(self, w, rect):
        self.width = rect.width
        self.height = rect.height

##----------------------------------------------------------------------

    def add_file_cb(self, w):

        dialog = OpenFileDialog(self, 1) # from utils module

        filename = dialog.filename
        files_list = dialog.files_list
        dir = os.path.dirname(filename) + os.sep

        if files_list:

            for f in files_list:
                path = dir+f
                try:
                    fd = open(path)
                except:
                    continue
                fd.close()

                book = self.text_buffer.new_book(path)
                self.bookmarks.add_new_book(book)

            self.update_treeview()
            self.emit('bookslist-changed')

##----------------------------------------------------------------------

    def add_category_cb(self, w, a=None):

        model = self.treeview.get_model()

        cat_name = entry_dialog(self, 'New category', 'Category: ')

        if cat_name:

            if self.bookmarks.categories_list.has_key(cat_name):
                error_dialog(self, 'Name must be unique')

            else:

                cat_iter = self.model.append(None)
                self.model.set(cat_iter, self.COLUMN_FOLDER, cat_name)
                self.bookmarks.make_new_category(cat_name)

                self.update_category_menu()

##----------------------------------------------------------------------

    def apply_cliced_cb(self, w):

        selected_list = self.get_selected_list()

        if len(selected_list) != 1:
            return

##         model, path, iter = selected_list[0]
##         filename = model.get_value(iter, self.COLUMN_PATH)

        self.change_apply()
        #self.emit('book-changed', filename) # ???

##----------------------------------------------------------------------

    def filters_toggled(self, w, fltr):

        selected_list = self.get_selected_list()

        if len(selected_list) != 1:
            return

        model, path, iter = selected_list[0]
        filename = model.get_value(iter, self.COLUMN_PATH)
        b = self.bookmarks.get_book(filename)
        active = w.get_active()

        if b:
            if active and not fltr in b.internalfilters:
                    if b.internalfilters:
                        b.internalfilters.append(fltr)
                    else:
                        b.internalfilters = [fltr]
            elif b.internalfilters and fltr in b.internalfilters:
                    b.internalfilters.remove(fltr)

        #self.change_apply()
        #self.emit('book-changed', filename) # ???

##----------------------------------------------------------------------

    def ext_filter_clicked_cb(self, w):

        selected_list = self.get_selected_list()

        if len(selected_list) != 1:
            return

        command = entry_dialog(self, 'External command', 'External command: ')

        if command:

            model, path, iter = selected_list[0]
            filename = model.get_value(iter, self.COLUMN_PATH)

            b = self.bookmarks.get_book(filename)
            if b:
                b.externalfilter = command
                self.treeview_show_selected(selected_list[0])
                #self.update_preview()

##----------------------------------------------------------------------

    def change_apply(self):

        selected_list = self.get_selected_list()

        if len(selected_list) != 1:
            return

        model, path, iter = selected_list[0]

        filename = model.get_value(iter, self.COLUMN_PATH)
        mark = model.get_value(iter, self.COLUMN_MARK)

        b = self.bookmarks.get_book(filename)

        if b:

            if mark: # selected mark rows
                m = self.bookmarks.get_mark(b, int(mark))
                if m:

                    m.author = self.author_entry.get_text()
                    m.name = self.name_entry.get_text()
                    start, end = self.description_buffer.get_bounds()
                    m.description = self.description_buffer.get_text(start,
                                                                     end, TRUE)
                    self.update_treeview()
            else:

                b.author = self.author_entry.get_text()
                b.name = self.name_entry.get_text()
                start, end = self.description_buffer.get_bounds()
                b.description = self.description_buffer.get_text(start,
                                                                 end, TRUE)
                self.update_treeview()

        self.clear_view()

##----------------------------------------------------------------------

    def emit_load_file(self):

        selected_list = self.get_selected_list()

        if len(selected_list) == 0:
            return

        model, path, iter = selected_list[0]

        filename = model.get_value(iter, self.COLUMN_PATH)
        mark = model.get_value(iter, self.COLUMN_MARK)

        if mark:
            b = self.bookmarks.get_book(filename)
            if b:
                m = self.bookmarks.get_mark(b, int(mark))
                if m:
                    b.current_position = m.position
                    b.current_offset = m.offset
                    self.emit('load-file', filename)

        elif filename:
            self.emit('load-file', filename)


##----------------------------------------------------------------------

    def delete_clicked_cb(self, w):

        selected_list = self.get_selected_list()

        if len(selected_list) == 0:
            return

        # remove mark
        for sel_indx in selected_list:

            model, path, iter = sel_indx
            filename = model.get_value(iter, self.COLUMN_PATH)
            mark = model.get_value(iter, self.COLUMN_MARK)

            if mark: # mark
                self.bookmarks.remove_bookmark(filename, int(mark))

        # remove book
        for sel_indx in selected_list:

            model, path, iter = sel_indx
            filename = model.get_value(iter, self.COLUMN_PATH)
            mark = model.get_value(iter, self.COLUMN_MARK)

            if not mark and filename: # book

                if self.bookmarks.remove_book(filename):
                    self.emit('book-removed', filename)

        # remove category
        for sel_indx in selected_list:

            model, path, iter = sel_indx
            filename = model.get_value(iter, self.COLUMN_PATH)
            mark = model.get_value(iter, self.COLUMN_MARK)

            if not mark and not filename: # category

                cat = model.get_value(iter, self.COLUMN_FOLDER)

                if len(self.bookmarks.categories_list[cat]) != 0:
                    error_dialog(self, 'Category must be empty')
                else:
                    del self.bookmarks.categories_list[cat]
                    self.update_category_menu()

        self.clear_view()
        self.update_treeview()
        self.emit('bookmarks-changed', filename)


##----------------------------------------------------------------------

    def clear_view(self):

        self.file_label.set_text('')
        self.author_entry.set_text('')
        self.name_entry.set_text('')
        self.description_buffer.set_text('')
        buffer = self.text_view.get_buffer()
        buffer.set_text('')


##----------------------------------------------------------------------

    def close_clicked_cb(self, w):
        self.destroy()

##----------------------------------------------------------------------

    def update_category_menu(self):
        # update categories menu
        self.category_submenu = gtk.Menu()
        if len(self.categories_list) == 0: # add 'Default' item
            i = 'Default'
            item = gtk.MenuItem(i)
            self.category_submenu.append(item)
            item.connect_object('activate',
                                self.category_submenu_activate_cb, i)
            item.show()
        else:
            for i in self.categories_list:
                item = gtk.MenuItem(i)
                self.category_submenu.append(item)
                item.connect_object('activate',
                                    self.category_submenu_activate_cb, i)
                item.show()
        self.category_menu.set_menu(self.category_submenu)

##----------------------------------------------------------------------

    def update_treeview(self):

        model = gtk.TreeStore(gobject.TYPE_STRING,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING)
        self.model = model

        if len(self.categories_list) > 0:
            for cat in self.categories_list: # categories
                iter = model.append(None)
                model.set(iter, self.COLUMN_FOLDER, cat)

                for book in self.categories_list[cat]: # books
                    book_iter = model.append(iter)

                    desc = fix_description(book.description, 100)
                    model.set(book_iter,
                              self.COLUMN_FILENAME, os.path.basename(book.filename),
                              self.COLUMN_NAME, book.name,
                              self.COLUMN_AUTHOR, book.author,
                              self.COLUMN_PERCENT, str(int(book.current_position*100)),
                              self.COLUMN_DECRIPTION, desc,
                              self.COLUMN_PATH, book.filename)

                    for mark in book.marks_list: # marks
                        mark_iter = model.append(book_iter)
                        desc = fix_description(mark.description, 100)
                        model.set(mark_iter,
                                  self.COLUMN_NAME, mark.name,
                                  self.COLUMN_AUTHOR, mark.author,
                                  self.COLUMN_PERCENT, str(int(mark.position*100)),
                                  self.COLUMN_DECRIPTION, desc,
                                  self.COLUMN_PATH, book.filename,
                                  self.COLUMN_MARK, mark.number)

        self.treeview.set_model(model)

        self.treeview.expand_all()



##----------------------------------------------------------------------

gobject.signal_new('load-file', BookmarksWindow,
                   gobject.SIGNAL_RUN_LAST,
                   gobject.TYPE_NONE,
                   (gobject.TYPE_STRING,))

gobject.signal_new('book-removed', BookmarksWindow,
                   gobject.SIGNAL_RUN_LAST,
                   gobject.TYPE_NONE,
                   (gobject.TYPE_STRING,))

gobject.signal_new('book-changed', BookmarksWindow,
                   gobject.SIGNAL_RUN_LAST,
                   gobject.TYPE_NONE,
                   (gobject.TYPE_STRING,))

gobject.signal_new('bookmarks-changed', BookmarksWindow,
                   gobject.SIGNAL_RUN_LAST,
                   gobject.TYPE_NONE,
                   (gobject.TYPE_STRING,))

gobject.signal_new('bookslist-changed', BookmarksWindow,
                   gobject.SIGNAL_RUN_LAST,
                   gobject.TYPE_NONE,
                   ())

##----------------------------------------------------------------------

if(__name__=='__main__'):
    # FIXME: sys.path
    bookmarks = Bookmarks()
    bw = BookmarksWindow(bookmarks)
    bw.set_default_size(600, 600)
    bw.paned.set_position(300)
    bw.connect('destroy', lambda w: gtk.main_quit())
    bw.show_all()
    gtk.main()

