#!/usr/bin/env python
# -*- mode: python; coding: koi8-r -*-

import gtk
from gtk import TRUE, FALSE

import string, gzip
import formatter, htmllib

from pybookreader.utils import string_encode, \
     default_description_length, default_encoding
from pybookreader.bookmarks import Book
from pybookreader.detectcharset import detectcharset
import pybookreader

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class basic_parser:
    def new_book(self, filename):
        """Create new book object"""
        pass
    def load_file(self, buffer, book):
        """Load file in buffer"""
        pass
    def preview(self, buffer, book, offset):
        """Create preview: load part of file in buffer"""
        pass
    def book_description(self, book):
        """Create description of book"""
        pass
    def mark_description(self, book, offset): # ??? (not used)
        """Create description of mark"""
        pass

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class plain_text:

    def __init__(self):
        pass

##----------------------------------------------------------------------

    def new_book(self, filename):
        """Create new book object"""

        book = Book()
        book.filename = filename
        encoding = detectcharset(filename)
        if encoding:
            book.encoding = encoding
        else:
            book.encoding = default_encoding
        book.file_type = 'plain_text'
        book.description = self.book_description(book)

        return book

##----------------------------------------------------------------------

    def load_file(self, buffer, book):
        """Load file in buffer"""

        data = self._read_file(filename=book.filename,
                               int_filters=book.internalfilters,
                               ext_filter=book.externalfilter,
                               charset=book.encoding)

        # delete invalid (\0) chars
        tbl = string.maketrans('', '')
        data = string.translate(data, tbl, '\0')

        data = string_encode(data, book.encoding, 'utf-8')

        buffer.set_text(data)

    def _read_file(self, filename, charset, length=None, offset=None,
                   int_filters=None, ext_filter=None):

        if ext_filter:

            import os
            command = ext_filter + ' ' + filename
            pipe = os.popen(command)
            data = pipe.read()
            pipe.close()
            if offset:
                data = data[offset:]
            if length:
                data = data[:length]

        else:

            fd = open(filename)
            if offset:
                fd.seek(offset)
            if length:
                data = fd.read(length)
            else:
                data = fd.read()
            fd.close()

        if int_filters:
            for fltr in int_filters:
                exec 'data = pybookreader.filters.%s (data, \'%s\')' \
                     % (fltr, charset)

        return data

##----------------------------------------------------------------------

    def preview(self, buffer, book, offset):
        """Create preview: load part of file in buffer"""

        data = self._read_file(filename=book.filename, length=1000,
                               offset=offset,
                               charset=book.encoding)
        data = string_encode(data, book.encoding, 'utf-8')
        buffer.set_text(data)


##----------------------------------------------------------------------

    def book_description(self, book):
        """Create description of book"""

        data = self._read_file(filename=book.filename, charset=book.encoding,
                               length=default_description_length)

        data = data.decode(book.encoding, 'replace')

        tbl = string.maketrans('', '')
        desc = string.translate(data.encode('utf-8', 'replace'), tbl, '\0')

        desc += ' ...'

        return desc


##----------------------------------------------------------------------

    def mark_description(self, book, offset): # ??? (not used)
        """Create description of mark"""
        print 'mark_description'

        data = self._read_file(filename=book.filename, charset=book.encoding)
        data = data[offset:offset+default_description_length]
        data = data.decode(book.encoding, 'replace')

        tbl = string.maketrans('', '')
        desc = string.translate(data.encode('utf-8', 'replace'), tbl, '\0')

        desc += ' ...'

        return desc

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class gzipped_plain_text(plain_text):

    def __init__(self):
        pass
        #plain_text.__init__(self)

    def _read_file(self, filename, charset, length=None, offset=None,
                   int_filters=None, ext_filter=None):

        fd = gzip.open(filename)
        if offset:
            fd.seek(offset)
        if length:
            data = fd.read(length)
        else:
            data = fd.read()
        fd.close()

        if int_filters:
            for fltr in int_filters:
                exec 'data = pybookreader.filters.%s (data, \'%s\')' \
                     % (fltr, charset)

        return data

##----------------------------------------------------------------------

    def new_book(self, filename):
        """Create new book object"""

        book = Book()
        book.filename = filename
        book.encoding = 'koi8-r' # FIXME
        book.file_type = 'gzipped_plain_text'
        book.description = self.book_description(book)

        return book

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class MyDumbWriter(formatter.NullWriter):

    def __init__(self, file=None, maxcol=72):
        self.data = []
        formatter.NullWriter.__init__(self)

    def send_paragraph(self, blankline):
        self.data.append('\n'*blankline)

    def send_line_break(self):
        self.data.append('\n')

    def send_hor_rule(self, *args, **kw):
        self.data.append('\n')
        #self.data.append('\n')

    def send_literal_data(self, data):
        self.data.append(data)

    def send_flowing_data(self, data):
        if not data: return
        self.data.append(data)


##----------------------------------------------------------------------

class html_parser(plain_text):

    def new_book(self, filename):
        """Create new book object"""

        book = Book()
        book.filename = filename
        encoding = detectcharset(filename)
        if encoding:
            book.encoding = encoding
        else:
            book.encoding = default_encoding
        book.file_type = 'html_parser'
        book.description = self.book_description(book)

        return book

##----------------------------------------------------------------------

    def _read_file(self, filename, charset, length=None, offset=None,
                   int_filters=None, ext_filter=None):

        ex = open(filename).read()
        w = MyDumbWriter()
        f = formatter.AbstractFormatter(w)
        h = htmllib.HTMLParser(f)
        h.feed(ex)
        data = string.join(w.data, '')
        if len(h.anchorlist) > 0:
            data += '\n\nLinks:\n'
            n = 1
            for i in h.anchorlist:
                data += '[%d] %s\n' % (n, i)
                n += 1

        if offset:
            data = data[offset:]
        if length:
            data = data[:length]

        if int_filters:
            for fltr in int_filters:
                exec 'data = pybookreader.filters.%s (data, \'%s\')' \
                     % (fltr, charset)

        return data

##----------------------------------------------------------------------

